//------------------------------------------------------------------------------
//
//  scene.cpp
//
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//  include
//------------------------------------------------------------------------------
#include "common.h"

static LPDIRECT3DVERTEXSHADER9  vshader = NULL;
static LPDIRECT3DPIXELSHADER9   pshader = NULL;
static LPCSTR filename = TEXT("main.fx");
static LPCSTR str_vs = "vs_main";
static LPCSTR str_ps = "ps_main";

//shader constant
static D3DXVECTOR4 shader_const[256];

//------------------------------------------------------------------------------
//  variable
//------------------------------------------------------------------------------
struct d3dverts {
  FLOAT x, y, z;
  FLOAT u, v;
  enum { fvf = D3DFVF_XYZ | D3DFVF_TEX1 };
};

static const d3dverts quadverts[4] = {
 { 1.0f, -1.0f, 0.0f,  1.0f, -1.0f},
 {-1.0f, -1.0f, 0.0f, -1.0f, -1.0f},
 { 1.0f,  1.0f, 0.0f,  1.0f,  1.0f},
 {-1.0f,  1.0f, 0.0f, -1.0f,  1.0f},
};

//------------------------------------------------------------------------------
//
//  function definition
//
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//  InitScene
//------------------------------------------------------------------------------
void InitScene() {
  HRESULT hRet = E_FAIL;
  LPD3DXBUFFER tmp;
  LPD3DXBUFFER err;
  TermScene();
  
  hRet = D3DXCompileShaderFromFile( filename,  0, 0, str_vs, "vs_3_0", D3DXSHADER_OPTIMIZATION_LEVEL3|D3DXSHADER_PREFER_FLOW_CONTROL, &tmp, &err, 0 );
  if(hRet != D3D_OK) {
    printf("Error : %s\n", err->GetBufferPointer());
    return;
  }
  hRet = d3ddevice->CreateVertexShader( (DWORD*)tmp->GetBufferPointer(), &vshader );
  if(hRet != D3D_OK) {
    printf("Error : %s\n", err->GetBufferPointer());
    return;
  }
  RELEASE(tmp);

  //hRet = D3DXCompileShaderFromFile( filename,  0, 0, str_ps, "ps_3_0", 0, &tmp, &err, 0 );
  hRet = D3DXCompileShaderFromFile( filename,  0, 0, str_ps, "ps_3_0", D3DXSHADER_OPTIMIZATION_LEVEL3|D3DXSHADER_PREFER_FLOW_CONTROL, &tmp, &err, 0 );
  if(hRet != D3D_OK) {
    printf("Error : %s\n", err->GetBufferPointer());
    return;
  }
  hRet = d3ddevice->CreatePixelShader( (DWORD*)tmp->GetBufferPointer(), &pshader );
  if(hRet != D3D_OK) {
    printf("Error : %s\n", err->GetBufferPointer());
    return;
  }
  RELEASE(tmp);
  printf("Load Shader.\n");
  
  
  d3ddevice->SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
  d3ddevice->SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
  d3ddevice->SetSamplerState(0, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR);
  d3ddevice->SetSamplerState(0, D3DSAMP_ADDRESSU, D3DTADDRESS_WRAP);
  d3ddevice->SetSamplerState(0, D3DSAMP_ADDRESSV, D3DTADDRESS_WRAP);
}

//------------------------------------------------------------------------------
//  TermScene
//------------------------------------------------------------------------------
void TermScene() {
  RELEASE(vshader);
  RELEASE(pshader);
}

//------------------------------------------------------------------------------
//  DoScene
//------------------------------------------------------------------------------
void DoScene() {
  if(GetAsyncKeyState(VK_F5)) {
    InitScene();
  }
  
  d3ddevice->Clear( 0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER|D3DCLEAR_STENCIL, 0, 1.0f, 0);
  if(d3ddevice->BeginScene() == D3D_OK) {
    
    //Setup shader.
    if(vshader) {
      d3ddevice->SetVertexShaderConstantF(0, (FLOAT*)&shader_const, 64);
      d3ddevice->SetVertexShader( vshader );
    } else {
      printf("Error : Vshader\n");
    }
    
    if(pshader) {
      d3ddevice->SetPixelShaderConstantF(0, (FLOAT*)&shader_const, 64);
      d3ddevice->SetPixelShader( pshader );
    } else {
      printf("Error : Pshader\n");
    }
    shader_const[0].x += 0.016666f;
    
    //Render
    d3ddevice->SetFVF( d3dverts::fvf  );
    d3ddevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, quadverts, sizeof(d3dverts));
    d3ddevice->EndScene();
  }
  d3ddevice->Present( NULL, NULL, NULL, NULL );
}

